﻿/* 
    Copyright Paul van Brenk (c) 2008
    More info @ http://www.codeplex.com/GoogleChartNet 
*/

using System;
using System.Web.UI.WebControls;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Drawing;
using System.Diagnostics;
using System.Collections.ObjectModel;
using System.Text;
using System.ComponentModel;

namespace Google.Charts
{
    /// <summary>
    /// Represents the base class for a chart.
    /// </summary>
    /// <typeparam name="T">The type of <see cref="DataPoint"/> the class supports.</typeparam>
    public abstract class Chart<T> : System.Web.UI.WebControls.Image
        where T : DataPoint/*, new() */
    {

        /// <summary>
        /// Initializes a new instance of the <see cref="Chart&lt;T&gt;"/> class.
        /// </summary>
        protected Chart()
            : base()
        {
            this.Resolution = Resolution.Low;
            this.Data = new List<DataSet<T>>();
        }


        /// <summary>
        /// Creates the size parameter.
        /// </summary>
        protected virtual void CreateSizeParameter()
        {
            ChartParameter size = new ChartParameter("chs", String.Format("{0}x{1}", Width.Value.ToString(), Height.Value.ToString()));
            parameters.Add(size);
        }

        ///todo: change comment
        /// <summary>
        /// Called by the ASP.NET page framework to notify server controls that use composition-based implementation to create any child controls they contain in preparation for posting back or rendering.
        /// </summary>
        protected abstract void CreateParameters();

        /// <summary>
        /// Creates the title parameter.
        /// </summary>
        protected virtual void CreateTitleParameter()
        {
            if (Title == null)
            {
                return;
            }

            parameters.Add(Title);

            //if (Title.Color.HasValue)
            //{
            //    parameters.Add(new ChartParameter("chts", Utils.ToHtml(Title.Color) + "," + Title.FontSize));

            //}
        }

        [Obsolete("change databinding")]
        protected DataSet<T> GetFirstDS()
        {
            DataSet<T> ds = null;
            if (Data.Count == 0)
            {
                ds = new DataSet<T>();
                Data.Add(ds);
            }
            else
                ds = Data[0];

            return ds;
        }

        /// <summary>
        /// Creates the background parameter.
        /// </summary>
        protected virtual void CreateBackgroundParameter()
        {
            string result = "";

            if (this.BackgroundFill != null)
            {
                result = "bg," + this.BackgroundFill.ToString();
            }

            ///todo: move this to charts supporting this
            if (this.ChartAreaFill != null)
            {
                if (result.Length > 0) { result += "|"; }
                result += "c," + this.ChartAreaFill.ToString();
            }

            if (result.Length == 0) { return; }


            ChartParameter param = new ChartParameter("chf", result);
            parameters.Add(param);
        }

        /// <summary>
        /// Generates the image URL.
        /// </summary>
        /// <returns></returns>
        protected virtual string GenerateImageUrl()
        {
            EnsureParameters();

            StringBuilder url = new StringBuilder(BaseUrl);

            // one & too many
            StringBuilder s = parameters.Aggregate(url, (x, y) => x.Append(y.ToQueryStringField() + "&"));
            
            return s.ToString();
        }

        protected void EnsureParameters()
        {
            if (parameters.Count == 0)
            {
                // allow derived controls to add their parameters
                CreateParameters();

                // add our minimal parameters, which does nothing if
                // the parametes is already in the parameter collection.
                CreateSizeParameter();
                CreateTitleParameter();
                CreateBackgroundParameter();
            }
        }


        /// <summary>
        /// Gets and sets the title of the chart.
        /// </summary>
        /// <value>The title of the chart.</value>
        /// <remarks>Use a pipe character (|) to force a line break.</remarks>
        public ChartTitle Title
        {
            get;
            set;
        }


        /// <summary>
        /// Gets or sets the data.
        /// </summary>
        /// <value>The data.</value> 
        public IList<DataSet<T>> Data
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets or sets the background fill.
        /// </summary>
        /// <value>The background fill.</value>
        public ChartFill BackgroundFill
        {
            get;
            set;
        }


        // only supported in line and scatterplot, move there
        [Obsolete]
        public virtual ChartFill ChartAreaFill
        {
            get;
            set;
        }

        public override string ImageUrl
        {
            get
            {
                return GenerateImageUrl();
            }
            set
            {
                throw new NotSupportedException("Url is generated based on the supplied properties.");
            }
        }

        /// <summary>
        /// Gets or sets the encoding resolution.
        /// </summary>
        /// <value>The encoding resolution.</value>
        public Resolution Resolution
        {
            get;
            set;
        }


        // the base of the url used to generate the graphs.
        private const string BaseUrl = "http://chart.apis.google.com/chart?";

        private HashSet<ChartParameter> parameters = new HashSet<ChartParameter>();


        /// <summary>
        /// Adds a parameter to the parameter collection of the chart.
        /// </summary>
        /// <param name="parameter">The parameter.</param>
        protected void AddParameter(ChartParameter parameter)
        {

            if (parameter == null)
            {
                throw new ArgumentNullException("parameter");
            }

            parameters.Add(parameter);
        }
    }

    public abstract class ColoredChart<T> : Chart<T>
        where T : DataPoint/*, new()*/
    {
        private List<Color> _colors = new List<Color>();

        public IList<Color> Colors
        {
            get { return _colors; }
        }

        protected override void CreateParameters()
        {
            StringBuilder sb = new StringBuilder();

            foreach (Color c in _colors)
            {
                sb.Append(Utils.ToHtml(c)).Append(",");
            }

            if (sb.Length == 0)
                return;
            else
                sb.Length--;

            ChartParameter colors = new ChartParameter("chco", sb.ToString());

            base.AddParameter(colors);
        }
    }
}
