﻿/* 
    Copyright Paul van Brenk (c) 2008
    More info @ http://www.codeplex.com/GoogleChartNet 
*/


using System;

namespace Google.Charts {

    /// <summary>
    /// The base-class for the encoders used to encode the data for the charts.
    /// </summary>
    public abstract class DataEncoder {

        /// <summary>
        /// Initializes a new instance of the <see cref="DataEncoder"/> class.
        /// </summary>
        protected DataEncoder() : this("") { }

        /// <summary>
        /// Initializes a new instance of the <see cref="DataEncoder"/> class.
        /// </summary>
        /// <param name="separator">The separator between datapoints.</param>
        protected DataEncoder(string separator) {
            this.Separator = separator;
            this.SetSeparator = "|";
            this.MissingValue = "";
        }

        /// <summary>
        /// When implemented in a derived class. Encodes maps the value to the 
        /// range of the encoder and returns the encoded values as a string.
        /// </summary>
        /// <param name="value">The value to encode.</param>
        /// <param name="maxValue">The max value in the dataset.</param>
        /// <returns>The encoded value.</returns>
        public abstract string EncodeValue(double value, double maxValue, double minValue);

        public string EncodeValue(double value, double maxValue)
        {
            return EncodeValue(value, maxValue, 0);
        }

        /// <summary>
        /// Gets an encoder to encode the data.
        /// </summary>
        /// <param name="resolution">The resolution.</param>
        /// <returns>The requested encoder.</returns>
        public static DataEncoder GetEncoder(Resolution resolution) {

            switch (resolution) {
                case Resolution.High:
                    return extendedEncoder;
                case Resolution.Low:
                    return simpleEncoder;
                case Resolution.Medium:
                    return textEncoder;
                default:
                    throw new ArgumentOutOfRangeException("resolution", "Invalid encoding resolution.");
            }
        }

        /// <summary>
        /// Determines whether the specified value is valid.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>
        /// 	<see langword="true" /> if the specified value is valid; otherwise, <see langword="false" />.
        /// </returns>
        public virtual bool IsValid(double value) {
            return (value >= 0 && false == Double.IsNaN(value));
        }

        public string Separator { get; protected set; }
        public string SetSeparator { get; protected set; }
        public string MissingValue { get; protected set; }
        
        /// <summary>
        /// When implemented; gets the string used to identify the 
        /// encoding method in the querystring.
        /// </summary>
        public abstract string Identifier { get; }


        private static readonly DataEncoder textEncoder = new TextEncoder();
        public static DataEncoder TextEncoder {
            get {
                return textEncoder;
            }
        }
        
        private readonly static DataEncoder simpleEncoder = new SimpleEncoder();
        public static DataEncoder SimpleEncoder {
            get {
                return simpleEncoder;
            }
        }

        private readonly static DataEncoder extendedEncoder = new ExtendedEncoder();
        public static DataEncoder ExtendedEncoder {
            get {
                return extendedEncoder;
            }
        }
    }
}
